@@ -20,23 +20,22 @@ FETCH_ONLY_EXPIRED_USERS = getattr(settings, 'VKONTAKTE_GROUPS_MIGRATION_FETCH_O
 class WrongMembershipsAmmount(Exception):
     pass
 
-def update_group_users(migration):
+class EnteredMembersAreNotLeft(Exception):
+    pass
+
+def update_group_users(group):
     '''
     Fetch all users of group, make new m2m relations, remove old m2m relations
     '''
-    group = migration.group
-    if migration.next:
-        migration = group.migrations.latest('id')
-
     log.debug('Fetching users for the group "%s"' % group)
     ids = GroupMembership.objects.get_user_ids(group, unique=False)
     User.remote.fetch(ids=ids, only_expired=FETCH_ONLY_EXPIRED_USERS)
 
     # process entered and left users of the group
     # here is possible using relative migration.members_*_ids, but it's more bugless to use absolute values, calculated from group.users
-    ids_current = group.users.values_list('remote_id', flat=True)
-    ids_left = set(ids_current).difference(set(migration.members_ids))
-    ids_entered = set(migration.members_ids).difference(set(ids_current))
+    ids_current = group.users.through.objects.filter(group=group).values_list('user_id', flat=True)
+    ids_left = set(ids_current).difference(set(ids))
+    ids_entered = set(ids).difference(set(ids_current))
 
     log.debug('Adding %d new users to the group "%s"' % (len(ids_entered), group))
     group.users.through.objects.bulk_create([group.users.through(group=group, user_id=id) for id in ids_entered])
@@ -305,7 +304,7 @@ class GroupMigration(models.Model):
             self.update_users_memberships()
         except WrongMembershipsAmmount, e:
             log.warning(e)
-            GroupMembership.objects.fix_timeline(group=self)
+            GroupMembership.objects.fix_timeline(self.group)
 
     def compare_with_siblings(self):
         if self.hidden or not self.prev or self.members_count < 10000:
@@ -416,7 +415,7 @@ class GroupMigration(models.Model):
             # ensure entered users not in memberships now
             error_ids_count = GroupMembership.objects.get_user_ids(self.group).filter(user_id__in=self.members_entered_ids).count()
             if error_ids_count != 0:
-                raise Exception("Found %d just entered users, that still not left from the group %s at %s" % (error_ids_count, self.group, self.time))
+                raise EnteredMembersAreNotLeft("Found %d just entered users, that still not left from the group %s at %s" % (error_ids_count, self.group, self.time))
 
             # ensure left users in memberships now
             left_ids_count = GroupMembership.objects.get_user_ids(self.group).filter(user_id__in=self.members_left_ids).count()
@@ -442,24 +441,35 @@ class GroupMembershipManager(models.Manager):
         migr = group.migrations.latest('id')
 
         while True:
-            if migr.check_memberships_count():
+            if not migr or migr.check_memberships_count():
                 break
             else:
                 migr = migr.prev
 
-        self.clear_timeline_after(migr.group, migr.time)
+        self.clear_timeline_after_migration(group, migr)
 
         while True:
             try:
-                migr = migr.next
+                migr = migr.next if migr else group.migrations.order_by('time')[0]
                 migr.save_final()
                 migr.check_memberships_count()
+            except EnteredMembersAreNotLeft:
+                # go back and clear one more
+                migr = migr.prev.prev
+                self.clear_timeline_after_migration(group, migr)
             except AttributeError:
                 break
 
+
         migr = group.migrations.latest('id')
         log.info('%s: %s == %s' % (group, self.get_user_ids(group).count(), migr.members_count))
 
+    def clear_timeline_after_migration(self, group, migr):
+        if migr:
+            self.clear_timeline_after(group, migr.time)
+        else:
+            self.filter(group=group).delete()
+
     def clear_timeline_after(self, group, time):
         '''
         Method:
